@@ -24,11 +24,15 @@
 #
 ###############################################################################
 
+from __future__ import absolute_import
+
 from collections import deque
 
+import txaio
+txaio.use_asyncio()
+
 from autobahn.wamp import websocket
 from autobahn.websocket import protocol
-from autobahn.websocket import http
 
 try:
     import asyncio
@@ -41,7 +45,7 @@ except ImportError:
     from trollius import iscoroutine
     from trollius import Future
 
-from autobahn._logging import make_logger
+from autobahn.websocket.types import ConnectionDeny
 
 
 __all__ = (
@@ -51,7 +55,6 @@ __all__ = (
     'WebSocketAdapterFactory',
     'WebSocketServerFactory',
     'WebSocketClientFactory',
-
     'WampWebSocketServerProtocol',
     'WampWebSocketClientProtocol',
     'WampWebSocketServerFactory',
@@ -83,12 +86,12 @@ class WebSocketAdapterProtocol(asyncio.Protocol):
             peer = transport.get_extra_info('peername')
             try:
                 # FIXME: tcp4 vs tcp6
-                self.peer = "tcp:%s:%d" % (peer[0], peer[1])
+                self.peer = u"tcp:%s:%d" % (peer[0], peer[1])
             except:
                 # e.g. Unix Domain sockets don't have host/port
-                self.peer = "unix:{0}".format(peer)
+                self.peer = u"unix:{0}".format(peer)
         except:
-            self.peer = "?"
+            self.peer = u"?"
 
         self._connectionMade()
 
@@ -104,8 +107,6 @@ class WebSocketAdapterProtocol(asyncio.Protocol):
                 data = self.receive_queue.popleft()
                 if self.transport:
                     self._dataReceived(data)
-                else:
-                    print("WebSocketAdapterProtocol._consume: no transport")
             self._consume()
 
         self.waiter.add_done_callback(process)
@@ -174,6 +175,13 @@ class WebSocketAdapterProtocol(asyncio.Protocol):
         if yields(res):
             asyncio.async(res)
 
+    def get_channel_id(self):
+        """
+        Implements :func:`autobahn.wamp.interfaces.ITransport.get_channel_id`
+        """
+        # FIXME
+        raise Exception("transport channel binding not implemented for asyncio")
+
     def registerProducer(self, producer, streaming):
         raise Exception("not implemented")
 
@@ -192,10 +200,10 @@ class WebSocketServerProtocol(WebSocketAdapterProtocol, protocol.WebSocketServer
             res = self.onConnect(request)
             # if yields(res):
             #  res = yield from res
-        except http.HttpException as exc:
-            self.failHandshake(exc.reason, exc.code)
-        except Exception:
-            self.failHandshake(http.INTERNAL_SERVER_ERROR[1], http.INTERNAL_SERVER_ERROR[0])
+        except ConnectionDeny as e:
+            self.failHandshake(e.reason, e.code)
+        except Exception as e:
+            self.failHandshake("Internal server error: {}".format(e), ConnectionDeny.INTERNAL_SERVER_ERROR)
         else:
             self.succeedHandshake(res)
 
@@ -210,12 +218,15 @@ class WebSocketClientProtocol(WebSocketAdapterProtocol, protocol.WebSocketClient
         if yields(res):
             asyncio.async(res)
 
+    def startTLS(self):
+        raise Exception("WSS over explicit proxies not implemented")
+
 
 class WebSocketAdapterFactory(object):
     """
     Adapter class for asyncio-based WebSocket client and server factories.
     """
-    log = make_logger()
+    log = txaio.make_logger()
 
     def __call__(self):
         proto = self.protocol()
@@ -275,9 +286,8 @@ class WampWebSocketServerFactory(websocket.WampWebSocketServerFactory, WebSocket
     def __init__(self, factory, *args, **kwargs):
 
         serializers = kwargs.pop('serializers', None)
-        debug_wamp = kwargs.pop('debug_wamp', False)
 
-        websocket.WampWebSocketServerFactory.__init__(self, factory, serializers, debug_wamp=debug_wamp)
+        websocket.WampWebSocketServerFactory.__init__(self, factory, serializers)
 
         kwargs['protocols'] = self._protocols
 
@@ -301,9 +311,8 @@ class WampWebSocketClientFactory(websocket.WampWebSocketClientFactory, WebSocket
     def __init__(self, factory, *args, **kwargs):
 
         serializers = kwargs.pop('serializers', None)
-        debug_wamp = kwargs.pop('debug_wamp', False)
 
-        websocket.WampWebSocketClientFactory.__init__(self, factory, serializers, debug_wamp=debug_wamp)
+        websocket.WampWebSocketClientFactory.__init__(self, factory, serializers)
 
         kwargs['protocols'] = self._protocols
 
