@@ -19,7 +19,7 @@
 # FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
 # AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
 # LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
-# OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN
+# OUT OF OR IN CONNECT<ION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN
 # THE SOFTWARE.
 #
 ###############################################################################
@@ -53,27 +53,39 @@ class ComponentConfig(object):
     provided to the constructor of :class:`autobahn.wamp.protocol.ApplicationSession`.
     """
 
-    def __init__(self, realm=None, extra=None):
+    __slots__ = (
+        'realm',
+        'extra',
+        'keyring',
+    )
+
+    def __init__(self, realm=None, extra=None, keyring=None):
         """
+
         :param realm: The realm the session should join.
         :type realm: unicode
-
-        :param extra: Optional user-supplied object with extra
-            configuration. This can be any object you like, and is
-            accessible in your `ApplicationSession` subclass via
-            `self.config.extra`. `dict` is a good default choice.
+        :param extra: Optional user-supplied object with extra configuration.
+            This can be any object you like, and is accessible in your
+            `ApplicationSession` subclass via `self.config.extra`. `dict` is
+            a good default choice. Important: if the component is to be hosted
+            by Crossbar.io, the supplied value must be JSON serializable.
+        :type extra: arbitrary
+        :param keyring: A mapper from WAMP URIs to "from"/"to" Ed25519 keys. When using
+            WAMP end-to-end encryption, application payload is encrypted using a
+            symmetric message key, which in turn is encrypted using the "to" URI (topic being
+            published to or procedure being called) public key and the "from" URI
+            private key. In both cases, the key for the longest matching URI is used.
+        :type keyring: obj implementing IKeyRing
         """
-        realm = realm or u'default'
-        # FIXME
-        if six.PY2 and type(realm) == str:
-            realm = six.u(realm)
-        if type(realm) != six.text_type:
-            raise RuntimeError('"realm" must be of type Unicode - was {0}'.format(type(realm)))
+        assert(realm is None or type(realm) == six.text_type)
+        # assert(keyring is None or ...) # FIXME
+
         self.realm = realm
         self.extra = extra
+        self.keyring = keyring
 
     def __str__(self):
-        return "ComponentConfig(realm = {0}, extra = {1})".format(self.realm, self.extra)
+        return "ComponentConfig(realm=<{0}>, extra={1}, keyring={2})".format(self.realm, self.extra, self.keyring)
 
 
 class HelloReturn(object):
@@ -87,9 +99,20 @@ class Accept(HelloReturn):
     Information to accept a ``HELLO``.
     """
 
-    def __init__(self, authid=None, authrole=None, authmethod=None, authprovider=None):
+    __slots__ = (
+        'realm',
+        'authid',
+        'authrole',
+        'authmethod',
+        'authprovider',
+        'authextra',
+    )
+
+    def __init__(self, realm=None, authid=None, authrole=None, authmethod=None, authprovider=None, authextra=None):
         """
 
+        :param realm: The realm the client is joined to.
+        :type realm: unicode
         :param authid: The authentication ID the client is assigned, e.g. ``"joe"`` or ``"joe@example.com"``.
         :type authid: unicode
         :param authrole: The authentication role the client is assigned, e.g. ``"anonymous"``, ``"user"`` or ``"com.myapp.user"``.
@@ -98,29 +121,25 @@ class Accept(HelloReturn):
         :type authmethod: unicode
         :param authprovider: The authentication provider that was used to authenticate the client, e.g. ``"mozilla-persona"``.
         :type authprovider: unicode
+        :param authextra: Application-specific authextra to be forwarded to the client in `WELCOME.details.authextra`.
+        :type authextra: dict
         """
-        if six.PY2:
-            if type(authid) == str:
-                authid = six.u(authid)
-            if type(authrole) == str:
-                authrole = six.u(authrole)
-            if type(authmethod) == str:
-                authmethod = six.u(authmethod)
-            if type(authprovider) == str:
-                authprovider = six.u(authprovider)
-
+        assert(realm is None or type(realm) == six.text_type)
         assert(authid is None or type(authid) == six.text_type)
         assert(authrole is None or type(authrole) == six.text_type)
         assert(authmethod is None or type(authmethod) == six.text_type)
         assert(authprovider is None or type(authprovider) == six.text_type)
+        assert(authextra is None or type(authextra) == dict)
 
+        self.realm = realm
         self.authid = authid
         self.authrole = authrole
         self.authmethod = authmethod
         self.authprovider = authprovider
+        self.authextra = authextra
 
     def __str__(self):
-        return "Accept(authid = {0}, authrole = {1}, authmethod = {2}, authprovider = {3})".format(self.authid, self.authrole, self.authmethod, self.authprovider)
+        return "Accept(realm=<{0}>, authid=<{1}>, authrole=<{2}>, authmethod={3}, authprovider={4}, authextra={5})".format(self.realm, self.authid, self.authrole, self.authmethod, self.authprovider, self.authextra)
 
 
 class Deny(HelloReturn):
@@ -128,6 +147,11 @@ class Deny(HelloReturn):
     Information to deny a ``HELLO``.
     """
 
+    __slots__ = (
+        'reason',
+        'message',
+    )
+
     def __init__(self, reason=u"wamp.error.not_authorized", message=None):
         """
 
@@ -136,12 +160,6 @@ class Deny(HelloReturn):
         :param message: A human readable message (for logging purposes).
         :type message: unicode
         """
-        if six.PY2:
-            if type(reason) == str:
-                reason = six.u(reason)
-            if type(message) == str:
-                message = six.u(message)
-
         assert(type(reason) == six.text_type)
         assert(message is None or type(message) == six.text_type)
 
@@ -149,7 +167,7 @@ class Deny(HelloReturn):
         self.message = message
 
     def __str__(self):
-        return "Deny(reason = {0}, message = '{1}')".format(self.reason, self.message)
+        return "Deny(reason=<{0}>, message='{1}')".format(self.reason, self.message)
 
 
 class Challenge(HelloReturn):
@@ -157,6 +175,11 @@ class Challenge(HelloReturn):
     Information to challenge the client upon ``HELLO``.
     """
 
+    __slots__ = (
+        'method',
+        'extra',
+    )
+
     def __init__(self, method, extra=None):
         """
 
@@ -167,15 +190,14 @@ class Challenge(HelloReturn):
            specific to the authentication method.
         :type extra: dict
         """
-        if six.PY2:
-            if type(method) == str:
-                method = six.u(method)
+        assert(type(method) == six.text_type)
+        assert(extra is None or type(extra) == dict)
 
         self.method = method
         self.extra = extra or {}
 
     def __str__(self):
-        return "Challenge(method = {0}, extra = {1})".format(self.method, self.extra)
+        return "Challenge(method={0}, extra={1})".format(self.method, self.extra)
 
 
 class HelloDetails(object):
@@ -183,25 +205,52 @@ class HelloDetails(object):
     Provides details of a WAMP session while still attaching.
     """
 
-    def __init__(self, roles=None, authmethods=None, authid=None, pending_session=None):
+    __slots__ = (
+        'realm',
+        'authmethods',
+        'authid',
+        'authrole',
+        'authextra',
+        'session_roles',
+        'pending_session',
+    )
+
+    def __init__(self, realm=None, authmethods=None, authid=None, authrole=None, authextra=None, session_roles=None, pending_session=None):
         """
 
-        :param roles: The WAMP roles and features supported by the attaching client.
-        :type roles: dict
+        :param realm: The realm the client wants to join.
+        :type realm: unicode or None
         :param authmethods: The authentication methods the client is willing to perform.
-        :type authmethods: list
-        :param authid: The authentication ID the client wants to authenticate as. Required for WAMP-CRA.
-        :type authid: str
+        :type authmethods: list of unicode or None
+        :param authid: The authid the client wants to authenticate as.
+        :type authid: unicode or None
+        :param authrole: The authrole the client wants to authenticate as.
+        :type authrole: unicode or None
+        :param authextra: Any extra information the specific authentication method requires the client to send.
+        :type authextra: arbitrary or None
+        :param session_roles: The WAMP session roles and features by the connecting client.
+        :type session_roles: dict or None
         :param pending_session: The session ID the session will get once successfully attached.
-        :type pending_session: int
+        :type pending_session: int or None
         """
-        self.roles = roles
+        assert(realm is None or type(realm) == six.text_type)
+        assert(authmethods is None or (type(authmethods) == list and all(type(x) == six.text_type for x in authmethods)))
+        assert(authid is None or type(authid) == six.text_type)
+        assert(authrole is None or type(authrole) == six.text_type)
+        assert(authextra is None or type(authextra) == dict)
+        # assert(session_roles is None or ...)  # FIXME
+        assert(pending_session is None or type(pending_session) in six.integer_types)
+
+        self.realm = realm
         self.authmethods = authmethods
         self.authid = authid
+        self.authrole = authrole
+        self.authextra = authextra
+        self.session_roles = session_roles
         self.pending_session = pending_session
 
     def __str__(self):
-        return "HelloDetails(roles = {0}, authmethods = {1}, authid = {2}, pending_session = {3})".format(self.roles, self.authmethods, self.authid, self.pending_session)
+        return "HelloDetails(realm=<{}>, authmethods={}, authid=<{}>, authrole=<{}>, authextra={}, session_roles={}, pending_session={})".format(self.realm, self.authmethods, self.authid, self.authrole, self.authextra, self.session_roles, self.pending_session)
 
 
 class SessionDetails(object):
@@ -211,35 +260,58 @@ class SessionDetails(object):
     .. seealso:: :func:`autobahn.wamp.interfaces.ISession.onJoin`
     """
 
-    def __init__(self, realm, session, authid=None, authrole=None, authmethod=None, authprovider=None):
+    __slots__ = (
+        'realm',
+        'session',
+        'authid',
+        'authrole',
+        'authmethod',
+        'authprovider',
+        'authextra',
+    )
+
+    def __init__(self, realm, session, authid=None, authrole=None, authmethod=None, authprovider=None, authextra=None):
         """
-        Ctor.
 
         :param realm: The realm this WAMP session is attached to.
         :type realm: unicode
         :param session: WAMP session ID of this session.
         :type session: int
         """
+        assert(type(realm) == six.text_type)
+        assert(type(session) in six.integer_types)
+        assert(authid is None or type(authid) == six.text_type)
+        assert(authrole is None or type(authrole) == six.text_type)
+        assert(authmethod is None or type(authmethod) == six.text_type)
+        assert(authprovider is None or type(authprovider) == six.text_type)
+        assert(authextra is None or type(authextra) == dict)
+
         self.realm = realm
         self.session = session
         self.authid = authid
         self.authrole = authrole
         self.authmethod = authmethod
         self.authprovider = authprovider
+        self.authextra = authextra
 
     def __str__(self):
-        return "SessionDetails(realm = {0}, session = {1}, authid = {2}, authrole = {3}, authmethod = {4})".format(self.realm, self.session, self.authid, self.authrole, self.authmethod)
+        return "SessionDetails(realm=<{0}>, session={1}, authid=<{2}>, authrole=<{3}>, authmethod={4}, authprovider={5}, authextra={6})".format(self.realm, self.session, self.authid, self.authrole, self.authmethod, self.authprovider, self.authextra)
 
 
 class CloseDetails(object):
     """
-    Provides details for a WAMP session upon open.
+    Provides details for a WAMP session upon close.
 
     .. seealso:: :func:`autobahn.wamp.interfaces.ISession.onLeave`
     """
     REASON_DEFAULT = u"wamp.close.normal"
     REASON_TRANSPORT_LOST = u"wamp.close.transport_lost"
 
+    __slots__ = (
+        'reason',
+        'message',
+    )
+
     def __init__(self, reason=None, message=None):
         """
 
@@ -248,11 +320,14 @@ class CloseDetails(object):
         :param message: Closing log message.
         :type message: unicode
         """
+        assert(reason is None or type(reason) == six.text_type)
+        assert(message is None or type(message) == six.text_type)
+
         self.reason = reason
         self.message = message
 
     def __str__(self):
-        return "CloseDetails(reason = {0}, message = '{1}'')".format(self.reason, self.message)
+        return "CloseDetails(reason=<{0}>, message='{1}')".format(self.reason, self.message)
 
 
 class SubscribeOptions(object):
@@ -261,8 +336,14 @@ class SubscribeOptions(object):
     :func:`autobahn.wamp.interfaces.ISubscriber.subscribe`.
     """
 
+    __slots__ = (
+        'match',
+        'details_arg',
+    )
+
     def __init__(self, match=None, details_arg=None):
         """
+
         :param match: The topic matching method to be used for the subscription.
         :type match: unicode
         :param details_arg: When invoking the handler, provide event details
@@ -276,13 +357,18 @@ class SubscribeOptions(object):
         self.details_arg = details_arg
 
     def message_attr(self):
-        # options dict as sent within WAMP message
-        return {
-            'match': self.match
-        }
+        """
+        Returns options dict as sent within WAMP messages.
+        """
+        options = {}
+
+        if self.match:
+            options[u'match'] = self.match
+
+        return options
 
     def __str__(self):
-        return "SubscribeOptions(match = {0}, details_arg = {1})".format(self.match, self.details_arg)
+        return "SubscribeOptions(match={0}, details_arg={1})".format(self.match, self.details_arg)
 
 
 class EventDetails(object):
@@ -290,23 +376,53 @@ class EventDetails(object):
     Provides details on an event when calling an event handler
     previously registered.
     """
-    def __init__(self, publication, publisher=None, topic=None):
+
+    __slots__ = (
+        'publication',
+        'publisher',
+        'publisher_authid',
+        'publisher_authrole',
+        'topic',
+        'enc_algo',
+    )
+
+    def __init__(self, publication, publisher=None, publisher_authid=None, publisher_authrole=None, topic=None, enc_algo=None):
         """
-        Ctor.
 
         :param publication: The publication ID of the event (always present).
         :type publication: int
         :param publisher: The WAMP session ID of the original publisher of this event.
-        :type publisher: int
+            Only filled when publisher is disclosed.
+        :type publisher: None or int
+        :param publisher_authid: The WAMP authid of the original publisher of this event.
+            Only filled when publisher is disclosed.
+        :type publisher_authid: None or unicode
+        :param publisher_authrole: The WAMP authrole of the original publisher of this event.
+            Only filled when publisher is disclosed.
+        :type publisher_authrole: None or unicode
         :param topic: For pattern-based subscriptions, the actual topic URI being published to.
-        :type topic1: unicode or None
+            Only filled for pattern-based subscriptions.
+        :type topic: None or unicode
+        :param enc_algo: Payload encryption algorithm that
+            was in use (currently, either `None` or `"cryptobox"`).
+        :type enc_algo: None or unicode
         """
+        assert(type(publication) in six.integer_types)
+        assert(publisher is None or type(publisher) in six.integer_types)
+        assert(publisher_authid is None or type(publisher_authid) == six.text_type)
+        assert(publisher_authrole is None or type(publisher_authrole) == six.text_type)
+        assert(topic is None or type(topic) == six.text_type)
+        assert(enc_algo is None or (type(enc_algo) == six.text_type and enc_algo in [u'cryptobox']))
+
         self.publication = publication
         self.publisher = publisher
+        self.publisher_authid = publisher_authid
+        self.publisher_authrole = publisher_authrole
         self.topic = topic
+        self.enc_algo = enc_algo
 
     def __str__(self):
-        return "EventDetails(publication = {0}, publisher = {1}, topic = {2})".format(self.publication, self.publisher, self.topic)
+        return "EventDetails(publication={0}, publisher={1}, publisher_authid={2}, publisher_authrole={3}, topic=<{4}>, enc_algo={5})".format(self.publication, self.publisher, self.publisher_authid, self.publisher_authrole, self.topic, self.enc_algo)
 
 
 class PublishOptions(object):
@@ -315,12 +431,26 @@ class PublishOptions(object):
     :func:`autobahn.wamp.interfaces.IPublisher.publish`.
     """
 
+    __slots__ = (
+        'acknowledge',
+        'exclude_me',
+        'exclude',
+        'exclude_authid',
+        'exclude_authrole',
+        'eligible',
+        'eligible_authid',
+        'eligible_authrole',
+    )
+
     def __init__(self,
                  acknowledge=None,
                  exclude_me=None,
                  exclude=None,
+                 exclude_authid=None,
+                 exclude_authrole=None,
                  eligible=None,
-                 disclose_me=None):
+                 eligible_authid=None,
+                 eligible_authrole=None):
         """
 
         :param acknowledge: If ``True``, acknowledge the publication with a success or
@@ -328,42 +458,72 @@ class PublishOptions(object):
         :type acknowledge: bool
         :param exclude_me: If ``True``, exclude the publisher from receiving the event, even
            if he is subscribed (and eligible).
-        :type exclude_me: bool
-        :param exclude: List of WAMP session IDs to exclude from receiving this event.
-        :type exclude: list of int
-        :param eligible: List of WAMP session IDs eligible to receive this event.
-        :type eligible: list of int
-        :param disclose_me: If ``True``, request to disclose the publisher of this event
-           to subscribers.
-        :type disclose_me: bool
+        :type exclude_me: bool or None
+        :param exclude: A single WAMP session ID or a list thereof to exclude from receiving this event.
+        :type exclude: int or list of int or None
+        :param exclude_authid: A single WAMP authid or a list thereof to exclude from receiving this event.
+        :type exclude_authid: unicode or list of unicode or None
+        :param exclude_authrole: A single WAMP authrole or a list thereof to exclude from receiving this event.
+        :type exclude_authrole: list of unicode or None
+        :param eligible: A single WAMP session ID or a list thereof eligible to receive this event.
+        :type eligible: int or list of int or None
+        :param eligible_authid: A single WAMP authid or a list thereof eligible to receive this event.
+        :type eligible_authid: unicode or list of unicode or None
+        :param eligible_authrole: A single WAMP authrole or a list thereof eligible to receive this event.
+        :type eligible_authrole: unicode or list of unicode or None
         """
-        # filter out None entries from exclude list, so it's easier for callers
-        if type(exclude) == list:
-            exclude = [x for x in exclude if x is not None]
         assert(acknowledge is None or type(acknowledge) == bool)
         assert(exclude_me is None or type(exclude_me) == bool)
-        assert(exclude is None or (type(exclude) == list and all(type(x) in six.integer_types for x in exclude)))
-        assert(eligible is None or (type(eligible) == list and all(type(x) in six.integer_types for x in eligible)))
-        assert(disclose_me is None or type(disclose_me) == bool)
+        assert(exclude is None or type(exclude) in six.integer_types or (type(exclude) == list and all(type(x) in six.integer_types for x in exclude)))
+        assert(exclude_authid is None or type(exclude_authid) == six.text_type or (type(exclude_authid) == list and all(type(x) == six.text_type for x in exclude_authid)))
+        assert(exclude_authrole is None or type(exclude_authrole) == six.text_type or (type(exclude_authrole) == list and all(type(x) == six.text_type for x in exclude_authrole)))
+        assert(eligible is None or type(eligible) in six.integer_types or (type(eligible) == list and all(type(x) in six.integer_types for x in eligible)))
+        assert(eligible_authid is None or type(eligible_authid) == six.text_type or (type(eligible_authid) == list and all(type(x) == six.text_type for x in eligible_authid)))
+        assert(eligible_authrole is None or type(eligible_authrole) == six.text_type or (type(eligible_authrole) == list and all(type(x) == six.text_type for x in eligible_authrole)))
 
         self.acknowledge = acknowledge
         self.exclude_me = exclude_me
         self.exclude = exclude
+        self.exclude_authid = exclude_authid
+        self.exclude_authrole = exclude_authrole
         self.eligible = eligible
-        self.disclose_me = disclose_me
+        self.eligible_authid = eligible_authid
+        self.eligible_authrole = eligible_authrole
 
     def message_attr(self):
-        # options dict as sent within WAMP message
-        return {
-            u'acknowledge': self.acknowledge,
-            u'exclude_me': self.exclude_me,
-            u'exclude': self.exclude,
-            u'eligible': self.eligible,
-            u'disclose_me': self.disclose_me
-        }
+        """
+        Returns options dict as sent within WAMP messages.
+        """
+        options = {}
+
+        if self.acknowledge:
+            options[u'acknowledge'] = self.acknowledge
+
+        if self.exclude_me:
+            options[u'exclude_me'] = self.exclude_me
+
+        if self.exclude:
+            options[u'exclude'] = self.exclude if type(self.exclude) == list else [self.exclude]
+
+        if self.exclude_authid:
+            options[u'exclude_authid'] = self.exclude_authid if type(self.exclude_authid) == list else self.exclude_authid
+
+        if self.exclude_authrole:
+            options[u'exclude_authrole'] = self.exclude_authrole if type(self.exclude_authrole) == list else self.exclude_authrole
+
+        if self.eligible:
+            options[u'eligible'] = self.eligible if type(self.eligible) == list else self.eligible
+
+        if self.eligible_authid:
+            options[u'eligible_authid'] = self.eligible_authid if type(self.eligible_authid) == list else self.eligible_authid
+
+        if self.eligible_authrole:
+            options[u'eligible_authrole'] = self.eligible_authrole if type(self.eligible_authrole) == list else self.eligible_authrole
+
+        return options
 
     def __str__(self):
-        return "PublishOptions(acknowledge = {0}, exclude_me = {1}, exclude = {2}, eligible = {3}, disclose_me = {4})".format(self.acknowledge, self.exclude_me, self.exclude, self.eligible, self.disclose_me)
+        return "PublishOptions(acknowledge={0}, exclude_me={1}, exclude={2}, exclude_authid={3}, exclude_authrole={4}, eligible={5}, eligible_authid={6}, eligible_authrole={7})".format(self.acknowledge, self.exclude_me, self.exclude, self.exclude_authid, self.exclude_authrole, self.eligible, self.eligible_authid, self.eligible_authrole)
 
 
 class RegisterOptions(object):
@@ -372,6 +532,12 @@ class RegisterOptions(object):
     :func:`autobahn.wamp.interfaces.ICallee.register`.
     """
 
+    __slots__ = (
+        'match',
+        'invoke',
+        'details_arg',
+    )
+
     def __init__(self, match=None, invoke=None, details_arg=None):
         """
 
@@ -388,14 +554,21 @@ class RegisterOptions(object):
         self.details_arg = details_arg
 
     def message_attr(self):
-        # options dict as sent within WAMP message
-        return {
-            u'match': self.match,
-            u'invoke': self.invoke
-        }
+        """
+        Returns options dict as sent within WAMP messages.
+        """
+        options = {}
+
+        if self.match:
+            options[u'match'] = self.match
+
+        if self.invoke:
+            options[u'invoke'] = self.invoke
+
+        return options
 
     def __str__(self):
-        return "RegisterOptions(match = {0}, invoke = {1}, details_arg = {2})".format(self.match, self.invoke, self.details_arg)
+        return "RegisterOptions(match={0}, invoke={1}, details_arg={2})".format(self.match, self.invoke, self.details_arg)
 
 
 class CallDetails(object):
@@ -404,23 +577,51 @@ class CallDetails(object):
     registered is being called and opted to receive call details.
     """
 
-    def __init__(self, progress=None, caller=None, procedure=None):
+    __slots__ = (
+        'progress',
+        'caller',
+        'caller_authid',
+        'caller_authrole',
+        'procedure',
+        'enc_algo',
+    )
+
+    def __init__(self, progress=None, caller=None, caller_authid=None, caller_authrole=None, procedure=None, enc_algo=None):
         """
-        Ctor.
 
         :param progress: A callable that will receive progressive call results.
         :type progress: callable
         :param caller: The WAMP session ID of the caller, if the latter is disclosed.
+            Only filled when caller is disclosed.
         :type caller: int
+        :param caller_authid: The WAMP authid of the original caller of this event.
+            Only filled when caller is disclosed.
+        :type caller_authid: None or unicode
+        :param caller_authrole: The WAMP authrole of the original caller of this event.
+            Only filled when caller is disclosed.
+        :type caller_authrole: None or unicode
         :param procedure: For pattern-based registrations, the actual procedure URI being called.
-        :type procedure: unicode or None
+        :type procedure: None or unicode
+        :param enc_algo: Payload encryption algorithm that
+            was in use (currently, either `None` or `"cryptobox"`).
+        :type enc_algo: None or string
         """
+        assert(progress is None or callable(progress))
+        assert(caller is None or type(caller) in six.integer_types)
+        assert(caller_authid is None or type(caller_authid) == six.text_type)
+        assert(caller_authrole is None or type(caller_authrole) == six.text_type)
+        assert(procedure is None or type(procedure) == six.text_type)
+        assert(enc_algo is None or (type(enc_algo) == six.text_type and enc_algo in [u'cryptobox']))
+
         self.progress = progress
         self.caller = caller
+        self.caller_authid = caller_authid
+        self.caller_authrole = caller_authrole
         self.procedure = procedure
+        self.enc_algo = enc_algo
 
     def __str__(self):
-        return "CallDetails(progress = {0}, caller = {1}, procedure = {2})".format(self.progress, self.caller, self.procedure)
+        return "CallDetails(progress={0}, caller={1}, caller_authid={2}, caller_authrole={3}, procedure=<{4}>, enc_algo={5})".format(self.progress, self.caller, self.caller_authid, self.caller_authrole, self.procedure, self.enc_algo)
 
 
 class CallOptions(object):
@@ -428,10 +629,14 @@ class CallOptions(object):
     Used to provide options for calling with :func:`autobahn.wamp.interfaces.ICaller.call`.
     """
 
+    __slots__ = (
+        'on_progress',
+        'timeout',
+    )
+
     def __init__(self,
                  on_progress=None,
-                 timeout=None,
-                 disclose_me=None):
+                 timeout=None):
         """
 
         :param on_progress: A callback that will be called when the remote endpoint
@@ -439,32 +644,29 @@ class CallOptions(object):
         :type on_progress: callable
         :param timeout: Time in seconds after which the call should be automatically canceled.
         :type timeout: float
-        :param disclose_me: Request to disclose the identity of the caller (it's WAMP session ID)
-           to Callees. Note that a Dealer, depending on Dealer configuration, might
-           reject the request, or might disclose the Callee's identity without
-           a request to do so.
-        :type disclose_me: bool
         """
         assert(on_progress is None or callable(on_progress))
         assert(timeout is None or (type(timeout) in list(six.integer_types) + [float] and timeout > 0))
-        assert(disclose_me is None or type(disclose_me) == bool)
 
         self.on_progress = on_progress
         self.timeout = timeout
-        self.disclose_me = disclose_me
 
     def message_attr(self):
-        # options dict as sent within WAMP message
-        res = {
-            u'timeout': self.timeout,
-            u'disclose_me': self.disclose_me
-        }
+        """
+        Returns options dict as sent within WAMP messages.
+        """
+        options = {}
+
+        if self.timeout:
+            options[u'timeout'] = self.timeout
+
         if self.on_progress:
-            res['receive_progress'] = True
-        return res
+            options[u'receive_progress'] = True
+
+        return options
 
     def __str__(self):
-        return "CallOptions(on_progress = {0}, timeout = {1}, disclose_me = {2})".format(self.on_progress, self.timeout, self.disclose_me)
+        return "CallOptions(on_progress={0}, timeout={1})".format(self.on_progress, self.timeout)
 
 
 class CallResult(object):
@@ -473,20 +675,29 @@ class CallResult(object):
     return values or keyword return values.
     """
 
+    __slots__ = (
+        'results',
+        'kwresults',
+        'enc_algo',
+    )
+
     def __init__(self, *results, **kwresults):
         """
-        Constructor.
 
         :param results: The positional result values.
         :type results: list
         :param kwresults: The keyword result values.
         :type kwresults: dict
         """
+        enc_algo = kwresults.pop('enc_algo', None)
+        assert(enc_algo is None or (type(enc_algo) == six.text_type and enc_algo in [u'cryptobox']))
+
+        self.enc_algo = enc_algo
         self.results = results
         self.kwresults = kwresults
 
     def __str__(self):
-        return "CallResult(results = {0}, kwresults = {1})".format(self.results, self.kwresults)
+        return "CallResult(results={0}, kwresults={1}, enc_algo={2})".format(self.results, self.kwresults, self.enc_algo)
 
 
 class IPublication(object):
